#!/bin/sh
#----------------------------------------------------------------------------
# dhcp2ens - dynamically configure ens (dns)
#
# Creation:    14.04.2002  jed
# Last Update: 05.03.2004  jed
#
# Copyright (c) 2000-2001 Frank Meyer
#----------------------------------------------------------------------------

leases_file_changed ()
{
    # update lease file marker
    if [ ! -f $last_checked_file -o \( $leases_file -nt $last_checked_file \) ]
    then
        > $last_checked_file
        return 0
    else
        return 1
    fi    
}

trfl_get_abs_time ()
{
    day=$1
    month=$2
    year=$3
    hour=$4
    minute=$5
    date -d "$year.$month.$day-$hour:$minute" +%s
}

backup_ens_conf ()
{
    if [ -f $ens_config_file ]
    then
        # backup existing config file
        cp $ens_config_file $ens_config_file.backup
    fi
}

backup_ens_listfile ()
{
    if [ -f $dhcp2ens_list_file ]
    then
        cp $dhcp2ens_list_file $dhcp2ens_list_file.backup
    fi
}

print_entry ()
{
    if [ -f $ens_config_file.new ]
    then
        # check if entry (ip address) already exists
        grep "$ipaddr " $ens_config_file.new > /dev/null

        if [ "$?" != "0" ]
        then
            # if not, check if entry (hostname) already exists
            grep -i "$host\." $ens_config_file.new > /dev/null

            if [ "$?" != "0" ]
            then
                # if not, add dynamic dns entry
                tmpipaddr=`echo $ipaddr|sed 's/\./ /g'`
                set $tmpipaddr
                rev=$4.$3.$2.$1

                domain=`grep "^SOA" $ens_config_file.base|cut -d' ' -f2`
                echo "$ipaddr $host.$domain"
                echo "PTR $rev.in-addr.arpa $host.$domain"
            fi
        fi
    fi
}

write_entry_to_listfile ()
{
    if [ -f $dhcp2ens_list_file.new ]
    then
        # check if entry (ip address) already exists
        grep "$ipaddr " $dhcp2ens_list_file.new > /dev/null

        if [ "$?" != "0" ]
        then
            # if not, check if entry (hostname) already exists
            grep -i "$host\." $dhcp2ens_list_file.new > /dev/null

            if [ "$?" != "0" ]
            then
                # if not, add dynamic dns entry
                echo "$ipaddr $host" >> $dhcp2ens_list_file.new
            fi
        fi
    else
        # file does not exist, create it
        echo "$ipaddr $host" >> $dhcp2ens_list_file.new
    fi
}

write_dynamic_ens_entries ()
{
    # remove temp file
    rm -f $leases_temp_file
    rm -f $dhcp2ens_list_file.new

    echo "# dynamic dns entries"

    now=`date +%s`
    cat $leases_file | \
    while read inline
    do
        case $inline in
            lease* )
                set $inline
                ipaddr=$2
                host='<unknown>'
                ;;

            ends* )
                set $inline
                end_day=$3
                end_time=$4
                ;;

            client-hostname* )
                set $inline
                host=`echo $2|sed -e 's/"\([^"]*\).*/\1/g'`
                ;;

            }* )
                OLDIFS=$IFS

                IFS='/'
                set $end_day
                year=$1
                month=$2
                day=$3

                IFS=':'
                set $end_time
                hour=$1
                minute=$2

                IFS=$OLDIFS

#               echo "ip: $ipaddr, ends: $end_day:$end_time, name: '$host'"
                exp=`trfl_get_abs_time $day $month $year $hour $minute`

                if [ $exp -ge $now ]
                then
                    # lease ok
                    print_entry
                    write_entry_to_listfile
#               else
#                   # lease expired
                fi
                ;;

            * )
                ;;
        esac
    done    
}

create_new_ens_conf ()
{
    echo "creating ENS configuration file ..."

    (
        # use base ens.conf
        cat $ens_config_file.base

        # add dynamic entries
        write_dynamic_ens_entries
    ) > $ens_config_file.new
}

activate_ens_conf ()
{
    if [ -f $ens_config_file.prev ]
    then
        # router currently online, process .prev file
        cp $ens_config_file.new $ens_config_file.prev

        grep -v forwarder $ens_config_file.new     > $ens_config_file
        grep    forwarder $ens_config_file.backup >> $ens_config_file
    else
        #  router offline
        cp $ens_config_file.new $ens_config_file
    fi

    # remove temp file
    rm -f $ens_config_file.new
}

activate_ens_listfile ()
{
    if [ -f $dhcp2ens_list_file.new ]
    then
        # replace file
        mv -f $dhcp2ens_list_file.new $dhcp2ens_list_file
    fi
}

force_ens_conf_reload ()
{
    pid=`ps -ef|grep \/ens|grep -v grep|tr -s ' ' ' '|cut -d' ' -f2`

    if [ "$pid" != "" ]
    then
        # force ens to reload config file
        kill -HUP $pid > /dev/null
    fi
}

#----------------------------------------------------------------------------
# main
#----------------------------------------------------------------------------

# set -x

dhcp2ens_config_file=/var/run/dhcp2ens.cfg
dhcp2ens_list_file=/var/run/dhcp_ip_names

if [ -f $dhcp2ens_config_file ]
then
    . $dhcp2ens_config_file

    # set variables
    ens_config_file=/usr/local/ens/ens.conf
    leases_file=$DHCP2ENS_LEASES_DIR/dhcpd.leases

    leases_temp_file=`dirname $ens_config_file`/`basename $leases_file.temp`
    last_checked_file=`dirname $ens_config_file`/dhcp2ens_last_checked

    loop_interval=$DHCP2ENS_INTERVAL

    if [ -f $ens_config_file.base ]
    then
        # file exists - function enabled
        while true
        do
            # check periodically
            if [ -f $leases_file ]
            then
                # leases file exists
                if leases_file_changed
                then
                    # leases file has been modified => go on
                    /usr/bin/logger -t "dhcp2ens" -p "local0.info" "updating ens.conf ..."
                    backup_ens_conf 
                    backup_ens_listfile 

                    create_new_ens_conf

                    activate_ens_conf
                    activate_ens_listfile

                    # set ownership
                    chown ens:ens $ens_config_file*

                    force_ens_conf_reload
                fi

                sleep $loop_interval
            else
                # lease file does not exist
                /usr/local/bin/colecho "dhcp2ens: no leases file found, exiting!" rd
                /usr/bin/logger -t "dhcp2ens" -p "local0.warning" "no leases file found, exiting!"
                break
            fi
        done
    fi    
fi

#set +x